<?php


/**
 * @file
 * Contains various relationship handlers.
 */

/**
 * Specialized relationship handler associating groups and their entity.
 *
 * @ingroup views
 */
class og_handler_relationship extends views_handler_relationship {

  /**
   * Add an extra option to filter by the field name.
   */
  function option_definition() {
    $options = parent::option_definition();

    $options['type'] = array('default' => FALSE);

    return $options;
  }

  /**
   * Expose our extra option for the field name.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    // This extra option is only useful when joining to the og_membership
    // table. In all other situations, a filer will do the same job.
    if ($this->definition['base'] == 'og_membership') {
      $wrapper = entity_metadata_wrapper('og_membership');
      $form['type'] = array(
        '#type' => 'select',
        '#title' => t('Type'),
        '#description' => t('Optionally filter by the type of OG Membership.'),
        '#options' => $wrapper->type->optionsList(),
        '#empty_value' => FALSE,
        '#default_value' => !empty($this->options['type']) ? $this->options['type']: FALSE,
      );
    }
  }

  /**
   * Called to implement a relationship in a query.
   */
  function query() {

    // If the entity type is specific for the og_membership
    // filter the join to select membership of those entity types.
    if (isset($this->definition['entity'])) {
      $extra = array(
        'field' => 'entity_type',
        'value' => $this->definition['entity'],
      );

      // Only add the table if og_membership is the left table since when the
      // table is specified to views_join, it only translates the table alias
      // of tables in the left position, however if no table is specified,
      // then views_join correctly inserts the alias of the table in the right
      // position of the join.
      if ($this->definition['base'] != 'og_membership') {
        $extra['table'] = 'og_membership';
      }

      $this->definition['extra'][] = $extra;
    }

    // If the group type is specific for the og_membership
    // filter the join to select membership of those group types.
    if (isset($this->definition['group_type'])) {
      $extra = array(
        'field' => 'group_type',
        'value' => $this->definition['group_type'],
      );

      // Only add the table if og_membership is the left table since when the
      // table is specified to views_join, it only translates the table alias
      // of tables in the left position, however if no table is specified,
      // then views_join correctly inserts the alias of the table in the right
      // position of the join.
      if ($this->definition['base'] != 'og_membership') {
        $extra['table'] = 'og_membership';
      }

      $this->definition['extra'][] = $extra;
    }

    if (!empty($this->options['type'])) {
      $this->definition['extra'][] = array(
        'field' => 'type',
        'value' => $this->options['type'],
      );
    }

    parent::query();
  }
}
