(function ($) {

/**
 * Adjust the behaviour of the datepicker so that:
 * - It doesn't close when you select a date
 * - Changing month or year select the date and update the value
 * - Closing triggers a change for other js behavior
 */
/**
 * Overrides the default _selectDay behavior to prevent the popup being closed.
 */
$.datepicker._selectDay = function(id, month, year, td) {
  // Copied over from the original function.
  var inst,
    target = $(id);

  if ($(td).hasClass(this._unselectableClass) || this._isDisabledDatepicker(target[0])) {
    return;
  }

  inst = this._getInst(target[0]);
  inst.selectedDay = inst.currentDay = $("a", td).html();
  inst.selectedMonth = inst.currentMonth = month;
  inst.selectedYear = inst.currentYear = year;

  // Update the input with the new value.
  target.val(this._formatDate(inst,
    inst.currentDay, inst.currentMonth, inst.currentYear));

  // Re-draw the popup so it shows the correct highlighted date.
  $.datepicker._updateDatepicker(inst);
};

/**
 * Override the default _hideDatepicker behavior to trigger a change event. We
 * maintain the processing in the original.
 */
$.datepicker._hideDatepickerOriginal = $.datepicker._hideDatepicker;
$.datepicker._hideDatepicker = function(input) {
  // Perform all the normal processing.
  $.datepicker._hideDatepickerOriginal(input);

  // Trigger the change on the input.
  this._curInst.input.trigger('change');
}

/**
 * Override the default _selectMonthYear behavior to update the selected date.
 * We maintain the processing in the original.
 */
$.datepicker._selectMonthYearOriginal = $.datepicker._selectMonthYear;
$.datepicker._selectMonthYear = function(id, select, period) {
  // Perform all the normal processing.
  $.datepicker._selectMonthYearOriginal(id, select, period);

  // Get hold of our target and instance.
  var target = $(id),
    inst = this._getInst(target[0]);

  // Update the current dates to the selected ones.
  inst.currentMonth = inst.selectedMonth;
  inst.currentYear = inst.selectedYear;

  // Update our input with the new value.
  inst.input.val(this._formatDate(inst,
    inst.currentDay, inst.currentMonth, inst.currentYear));

  // Re-draw the popup so it shows the correct highlighted date.
  $.datepicker._updateDatepicker(inst);
}

})(jQuery);
